@@ -6,13 +6,11 @@
 //! # Overview
 //! The [wasmedge-macro](https://crates.io/crates/wasmedge-macro) crate defines a group of procedural macros used by both [wasmedge-sdk](https://crates.io/crates/wasmedge-sdk) and [wasmedge-sys](https://crates.io/crates/wasmedge-sys) crates.
 
-#![feature(extend_one)]
-
 use proc_macro::TokenStream;
 use quote::quote;
-use syn::{parse_macro_input, parse_quote, spanned::Spanned, FnArg, Item, Pat, PatType};
+use syn::{parse_macro_input, parse_quote, spanned::Spanned, FnArg, Item, Pat, PatIdent, PatType};
 
-/// Expand a synchronous host function defined with `wasmedge-sdk` crate.
+/// Declare a native function that will be used to create a host function instance.
 #[proc_macro_attribute]
 pub fn host_function(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let body_ast = parse_macro_input!(item as Item);
@@ -33,12 +31,13 @@ fn expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStre
     let wrapper_fn_name_literal = wrapper_fn_name_ident.to_string();
     // arguments of wrapper function
     let wrapper_fn_inputs: syn::punctuated::Punctuated<FnArg, syn::token::Comma> = parse_quote!(
-        frame: &wasmedge_sdk::CallingFrame,
+        frame: wasmedge_sdk::CallingFrame,
         args: Vec<wasmedge_sdk::WasmValue>,
-        data: *mut std::os::raw::c_void
     );
     // return type of wrapper function
     let wrapper_fn_return = item_fn.sig.output.clone();
+    // visibility of wrapper function
+    let wrapper_visibility = item_fn.vis.clone();
 
     // * define the signature of inner function
     // name of inner function
@@ -55,7 +54,7 @@ fn expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStre
     let ret = match item_fn.sig.inputs.len() {
         2 => {
             quote!(
-                fn #wrapper_fn_name_ident (#wrapper_fn_inputs) #wrapper_fn_return {
+                # wrapper_visibility fn #wrapper_fn_name_ident (#wrapper_fn_inputs) #wrapper_fn_return {
                     // define inner function
                     fn #inner_fn_name_ident (#inner_fn_inputs) #inner_fn_return {
                         #inner_fn_block
@@ -64,76 +63,7 @@ fn expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStre
                     // create a Caller instance
                     let caller = Caller::new(frame);
 
-                    #inner_fn_name_ident(&caller, args)
-                }
-            )
-        }
-        3 => {
-            let data_arg = item_fn.sig.inputs.last().unwrap().clone();
-            let ty_ptr = match &data_arg {
-                FnArg::Typed(PatType { ref ty, .. }) => match **ty {
-                    syn::Type::Reference(syn::TypeReference { ref elem, .. }) => syn::TypePtr {
-                        star_token: parse_quote!(*),
-                        const_token: None,
-                        mutability: Some(parse_quote!(mut)),
-                        elem: elem.clone(),
-                    },
-                    syn::Type::Path(syn::TypePath { ref path, .. }) => match path.segments.last() {
-                        Some(segment) => {
-                            let id = segment.ident.to_string();
-                            match id == "Option" {
-                                true => match segment.arguments {
-                                    syn::PathArguments::AngleBracketed(
-                                        syn::AngleBracketedGenericArguments { ref args, .. },
-                                    ) => {
-                                        let last_generic_arg = args.last();
-                                        match last_generic_arg {
-                                            Some(arg) => match arg {
-                                                syn::GenericArgument::Type(ty) => match ty {
-                                                    syn::Type::Reference(syn::TypeReference {
-                                                        ref elem,
-                                                        ..
-                                                    }) => syn::TypePtr {
-                                                        star_token: parse_quote!(*),
-                                                        const_token: None,
-                                                        mutability: Some(parse_quote!(mut)),
-                                                        elem: elem.clone(),
-                                                    },
-                                                    _ => panic!("Not found syn::Type::Reference"),
-                                                },
-                                                _ => {
-                                                    panic!("Not found syn::GenericArgument::Type")
-                                                }
-                                            },
-                                            None => panic!("Not found the last GenericArgument"),
-                                        }
-                                    }
-                                    _ => panic!("Not found syn::PathArguments::AngleBracketed"),
-                                },
-                                false => panic!("Not found segment ident: Option"),
-                            }
-                        }
-                        None => panic!("Not found path segments"),
-                    },
-                    _ => panic!("Unsupported syn::Type type"),
-                },
-                _ => panic!("Unsupported syn::FnArg type"),
-            };
-
-            // generate token stream
-            quote!(
-                fn #wrapper_fn_name_ident (#wrapper_fn_inputs) #wrapper_fn_return {
-                    // define inner function
-                    fn #inner_fn_name_ident (#inner_fn_inputs) #inner_fn_return {
-                        #inner_fn_block
-                    }
-
-                    // create a Caller instance
-                    let caller = Caller::new(frame);
-
-                    let data = unsafe { &mut *(data as #ty_ptr) };
-
-                    #inner_fn_name_ident(&caller, args, data)
+                    #inner_fn_name_ident(caller, args)
                 }
             )
         }
@@ -145,10 +75,10 @@ fn expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStre
 
 #[doc(hidden)]
 #[proc_macro_attribute]
-pub fn async_host_function(_attr: TokenStream, item: TokenStream) -> TokenStream {
+pub fn async_host_function_original(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let body_ast = parse_macro_input!(item as Item);
     if let Item::Fn(item_fn) = body_ast {
-        match expand_async_host_func(&item_fn) {
+        match expand_async_host_func_original(&item_fn) {
             Ok(token_stream) => token_stream.into(),
             Err(err) => err.to_compile_error().into(),
         }
@@ -157,7 +87,7 @@ pub fn async_host_function(_attr: TokenStream, item: TokenStream) -> TokenStream
     }
 }
 
-fn expand_async_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStream> {
+fn expand_async_host_func_original(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStream> {
     let outer_fn_name_ident = &item_fn.sig.ident;
     let outer_fn_name_literal = outer_fn_name_ident.to_string();
     let outer_fn_inputs = &item_fn.sig.inputs;
@@ -253,12 +183,12 @@ fn expand_async_host_func2(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::To
     Ok(ret)
 }
 
-#[doc(hidden)]
+/// Declare a native async function that will be used to create an async host function instance.
 #[proc_macro_attribute]
-pub fn sys_async_host_function(_attr: TokenStream, item: TokenStream) -> TokenStream {
+pub fn async_host_function(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let body_ast = parse_macro_input!(item as Item);
     if let Item::Fn(item_fn) = body_ast {
-        match sys_async_expand_host_func(&item_fn) {
+        match expand_async_host_func(&item_fn) {
             Ok(token_stream) => token_stream.into(),
             Err(err) => err.to_compile_error().into(),
         }
@@ -267,7 +197,68 @@ pub fn sys_async_host_function(_attr: TokenStream, item: TokenStream) -> TokenSt
     }
 }
 
-fn sys_async_expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStream> {
+fn expand_async_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStream> {
+    // extract T from Option<&mut T>
+    let ret = match &item_fn.sig.inputs.len() {
+        2 => expand_async_host_func_with_two_args(item_fn),
+        _ => panic!("Invalid numbers of host function arguments"),
+    };
+
+    Ok(ret)
+}
+
+fn expand_async_host_func_with_two_args(item_fn: &syn::ItemFn) -> proc_macro2::TokenStream {
+    let fn_name_ident = &item_fn.sig.ident;
+    let fn_visibility = &item_fn.vis;
+
+    // * prepare the function arguments
+    let mut fn_inputs = item_fn.sig.inputs.clone();
+    // process the first argument
+    let mut original_first_arg_ident: syn::Ident =
+        syn::Ident::new("placeholder", proc_macro2::Span::call_site());
+    if let Some(first) = fn_inputs.first_mut() {
+        if let FnArg::Typed(PatType { pat, .. }) = first {
+            if let Pat::Ident(PatIdent { ident, .. }) = &**pat {
+                // extract the name from the first argument
+                original_first_arg_ident = ident.clone();
+            }
+        }
+
+        // replace the first argument
+        *first = parse_quote!(frame: wasmedge_sdk::CallingFrame);
+    }
+
+    // * prepare the function body
+    let fn_block = &item_fn.block;
+
+    // compose the final function
+    quote!(
+        #fn_visibility fn #fn_name_ident (#fn_inputs) -> Box<(dyn std::future::Future<Output = Result<Vec<WasmValue>, HostFuncError>> + Send + 'static)> {
+            Box::new(async move {
+                let #original_first_arg_ident = Caller::new(frame);
+                #fn_block
+            })
+        }
+    )
+}
+
+#[doc(hidden)]
+#[proc_macro_attribute]
+pub fn sys_async_host_function_original(_attr: TokenStream, item: TokenStream) -> TokenStream {
+    let body_ast = parse_macro_input!(item as Item);
+    if let Item::Fn(item_fn) = body_ast {
+        match sys_async_expand_host_func_original(&item_fn) {
+            Ok(token_stream) => token_stream.into(),
+            Err(err) => err.to_compile_error().into(),
+        }
+    } else {
+        TokenStream::new()
+    }
+}
+
+fn sys_async_expand_host_func_original(
+    item_fn: &syn::ItemFn,
+) -> syn::Result<proc_macro2::TokenStream> {
     let outer_fn_name_ident = &item_fn.sig.ident;
     let outer_fn_name_literal = outer_fn_name_ident.to_string();
     let outer_fn_inputs = &item_fn.sig.inputs;
@@ -314,10 +305,10 @@ fn sys_async_expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2:
 
 #[doc(hidden)]
 #[proc_macro_attribute]
-pub fn sys_async_host_function2(_attr: TokenStream, item: TokenStream) -> TokenStream {
+pub fn sys_async_host_function_switcher2(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let body_ast = parse_macro_input!(item as Item);
     if let Item::Fn(item_fn) = body_ast {
-        match sys_expand_async_host_func2(&item_fn) {
+        match sys_expand_async_host_func_switcher2(&item_fn) {
             Ok(token_stream) => token_stream.into(),
             Err(err) => err.to_compile_error().into(),
         }
@@ -326,7 +317,9 @@ pub fn sys_async_host_function2(_attr: TokenStream, item: TokenStream) -> TokenS
     }
 }
 
-fn sys_expand_async_host_func2(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStream> {
+fn sys_expand_async_host_func_switcher2(
+    item_fn: &syn::ItemFn,
+) -> syn::Result<proc_macro2::TokenStream> {
     let outer_fn_name_ident = &item_fn.sig.ident;
     let outer_fn_name_literal = outer_fn_name_ident.to_string();
     let outer_fn_inputs = &item_fn.sig.inputs;
@@ -363,7 +356,6 @@ fn sys_expand_async_host_func2(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2
     Ok(ret)
 }
 
-/// Expand a synchronous host function defined with `wasmedge-sys` crate.
 #[proc_macro_attribute]
 pub fn sys_host_function(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let body_ast = parse_macro_input!(item as Item);
@@ -384,6 +376,8 @@ fn sys_expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::Token
     let wrapper_fn_name_literal = wrapper_fn_name_ident.to_string();
     // return type of wrapper function
     let wrapper_fn_return = item_fn.sig.output.clone();
+    // visibility of wrapper function
+    let wrapper_fn_visibility = item_fn.vis.clone();
 
     // * define the signature of inner function
     // name of inner function
@@ -418,88 +412,16 @@ fn sys_expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::Token
     let ret = match item_fn.sig.inputs.len() {
         2 => {
             // insert the third argument
-            let mut wrapper_fn_inputs = item_fn.sig.inputs.clone();
-            wrapper_fn_inputs.push(parse_quote!(_data: *mut std::os::raw::c_void));
+            let wrapper_fn_inputs = item_fn.sig.inputs.clone();
 
             quote!(
-                fn #wrapper_fn_name_ident (#wrapper_fn_inputs) #wrapper_fn_return {
+                #wrapper_fn_visibility fn #wrapper_fn_name_ident (#wrapper_fn_inputs) #wrapper_fn_return {
                     // define inner function
                     fn #inner_fn_name_ident (#inner_fn_inputs) #inner_fn_return {
                         #inner_fn_block
                     }
 
-                    #inner_fn_name_ident(&#arg1, #arg2)
-                }
-            )
-        }
-        3 => {
-            let data_arg = item_fn.sig.inputs.last().unwrap().clone();
-            let ty_ptr = match &data_arg {
-                FnArg::Typed(PatType { ref ty, .. }) => match **ty {
-                    syn::Type::Reference(syn::TypeReference { ref elem, .. }) => syn::TypePtr {
-                        star_token: parse_quote!(*),
-                        const_token: None,
-                        mutability: Some(parse_quote!(mut)),
-                        elem: elem.clone(),
-                    },
-                    syn::Type::Path(syn::TypePath { ref path, .. }) => match path.segments.last() {
-                        Some(segment) => {
-                            let id = segment.ident.to_string();
-                            match id == "Option" {
-                                true => match segment.arguments {
-                                    syn::PathArguments::AngleBracketed(
-                                        syn::AngleBracketedGenericArguments { ref args, .. },
-                                    ) => {
-                                        let last_generic_arg = args.last();
-                                        match last_generic_arg {
-                                            Some(arg) => match arg {
-                                                syn::GenericArgument::Type(ty) => match ty {
-                                                    syn::Type::Reference(syn::TypeReference {
-                                                        ref elem,
-                                                        ..
-                                                    }) => syn::TypePtr {
-                                                        star_token: parse_quote!(*),
-                                                        const_token: None,
-                                                        mutability: Some(parse_quote!(mut)),
-                                                        elem: elem.clone(),
-                                                    },
-                                                    _ => panic!("Not found syn::Type::Reference"),
-                                                },
-                                                _ => {
-                                                    panic!("Not found syn::GenericArgument::Type")
-                                                }
-                                            },
-                                            None => panic!("Not found the last GenericArgument"),
-                                        }
-                                    }
-                                    _ => panic!("Not found syn::PathArguments::AngleBracketed"),
-                                },
-                                false => panic!("Not found segment ident: Option"),
-                            }
-                        }
-                        None => panic!("Not found path segments"),
-                    },
-                    _ => panic!("Unsupported syn::Type type"),
-                },
-                _ => panic!("Unsupported syn::FnArg type"),
-            };
-
-            // inputs of wrapper function
-            let mut wrapper_fn_inputs = item_fn.sig.inputs.clone();
-            wrapper_fn_inputs.pop();
-            wrapper_fn_inputs.push(parse_quote!(data: *mut std::os::raw::c_void));
-
-            // generate token stream
-            quote!(
-                fn #wrapper_fn_name_ident (#wrapper_fn_inputs) #wrapper_fn_return {
-                    // define inner function
-                    fn #inner_fn_name_ident (#inner_fn_inputs) #inner_fn_return {
-                        #inner_fn_block
-                    }
-
-                    let data = unsafe { &mut *(data as #ty_ptr) };
-
-                    #inner_fn_name_ident(&#arg1, #arg2, data)
+                    #inner_fn_name_ident(#arg1, #arg2)
                 }
             )
         }
@@ -508,3 +430,44 @@ fn sys_expand_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::Token
 
     Ok(ret)
 }
+
+#[proc_macro_attribute]
+pub fn sys_async_host_function(_attr: TokenStream, item: TokenStream) -> TokenStream {
+    let body_ast = parse_macro_input!(item as Item);
+    if let Item::Fn(item_fn) = body_ast {
+        match sys_expand_async_host_func(&item_fn) {
+            Ok(token_stream) => token_stream.into(),
+            Err(err) => err.to_compile_error().into(),
+        }
+    } else {
+        TokenStream::new()
+    }
+}
+
+fn sys_expand_async_host_func(item_fn: &syn::ItemFn) -> syn::Result<proc_macro2::TokenStream> {
+    // extract T from Option<&mut T>
+    let ret = match &item_fn.sig.inputs.len() {
+        2 => sys_expand_async_host_func_with_two_args(item_fn),
+        _ => panic!("Invalid numbers of host function arguments"),
+    };
+
+    Ok(ret)
+}
+
+fn sys_expand_async_host_func_with_two_args(item_fn: &syn::ItemFn) -> proc_macro2::TokenStream {
+    let fn_name_ident = &item_fn.sig.ident;
+    let fn_visibility = &item_fn.vis;
+    let fn_inputs = item_fn.sig.inputs.clone();
+
+    let fn_block = &item_fn.block;
+
+    let ret = quote!(
+        #fn_visibility fn #fn_name_ident (#fn_inputs) -> Box<(dyn std::future::Future<Output = Result<Vec<WasmValue>, HostFuncError>> + Send + 'static)> {
+            Box::new(async move {
+                #fn_block
+            })
+        }
+    );
+
+    ret
+}
