@@ -74,6 +74,12 @@ impl Module {
             })
             .collect()
     }
+
+    /// Provides a raw pointer to the inner ASTModule context.
+    #[cfg(feature = "ffi")]
+    pub fn as_ptr(&self) -> *const ffi::WasmEdge_FunctionTypeContext {
+        self.inner.0 as *const _
+    }
 }
 
 #[derive(Debug)]
@@ -237,6 +243,12 @@ impl<'module> ImportType<'module> {
         };
         c_name.to_string_lossy()
     }
+
+    /// Provides a raw pointer to the inner ImportType context.
+    #[cfg(feature = "ffi")]
+    pub fn as_ptr(&self) -> *const ffi::WasmEdge_ImportTypeContext {
+        self.inner.0 as *const _
+    }
 }
 
 #[derive(Debug)]
@@ -388,6 +400,12 @@ impl<'module> ExportType<'module> {
         };
         c_name.to_string_lossy()
     }
+
+    /// Provides a raw pointer to the inner ExportType context.
+    #[cfg(feature = "ffi")]
+    pub fn as_ptr(&self) -> *const ffi::WasmEdge_ExportTypeContext {
+        self.inner.0 as *const _
+    }
 }
 
 #[derive(Debug)]
@@ -407,7 +425,7 @@ mod tests {
     #[test]
     fn test_module_clone() {
         let path = std::path::PathBuf::from(env!("WASMEDGE_DIR"))
-            .join("bindings/rust/wasmedge-sys/tests/data/import.wasm");
+            .join("bindings/rust/wasmedge-sys/examples/data/import.wat");
 
         let result = Config::create();
         assert!(result.is_ok());
@@ -416,7 +434,7 @@ mod tests {
         assert!(config.bulk_memory_operations_enabled());
 
         // load module from file
-        let result = Loader::create(Some(config));
+        let result = Loader::create(Some(&config));
         assert!(result.is_ok());
         let loader = result.unwrap();
         let result = loader.from_file(path);
@@ -435,7 +453,7 @@ mod tests {
     #[test]
     fn test_module_import() {
         let path = std::path::PathBuf::from(env!("WASMEDGE_DIR"))
-            .join("bindings/rust/wasmedge-sys/tests/data/import.wasm");
+            .join("bindings/rust/wasmedge-sys/examples/data/import.wat");
 
         let result = Config::create();
         assert!(result.is_ok());
@@ -444,7 +462,7 @@ mod tests {
         assert!(config.bulk_memory_operations_enabled());
 
         // load module from file
-        let result = Loader::create(Some(config));
+        let result = Loader::create(Some(&config));
         assert!(result.is_ok());
         let loader = result.unwrap();
         let result = loader.from_file(path);
@@ -587,7 +605,7 @@ mod tests {
     #[test]
     fn test_module_export() {
         let path = std::path::PathBuf::from(env!("WASMEDGE_DIR"))
-            .join("bindings/rust/wasmedge-sys/tests/data/import.wasm");
+            .join("bindings/rust/wasmedge-sys/examples/data/import.wat");
 
         let result = Config::create();
         assert!(result.is_ok());
@@ -596,7 +614,7 @@ mod tests {
         assert!(config.bulk_memory_operations_enabled());
 
         // load module from file
-        let result = Loader::create(Some(config));
+        let result = Loader::create(Some(&config));
         assert!(result.is_ok());
         let loader = result.unwrap();
         let result = loader.from_file(path);
@@ -737,7 +755,7 @@ mod tests {
     #[test]
     fn test_module_send() {
         let path = std::path::PathBuf::from(env!("WASMEDGE_DIR"))
-            .join("bindings/rust/wasmedge-sys/tests/data/import.wasm");
+            .join("bindings/rust/wasmedge-sys/examples/data/import.wat");
 
         let result = Config::create();
         assert!(result.is_ok());
@@ -746,7 +764,7 @@ mod tests {
         assert!(config.bulk_memory_operations_enabled());
 
         // load module from file
-        let result = Loader::create(Some(config));
+        let result = Loader::create(Some(&config));
         assert!(result.is_ok());
         let loader = result.unwrap();
         let result = loader.from_file(path);
@@ -891,7 +909,7 @@ mod tests {
     #[test]
     fn test_module_sync() {
         let path = std::path::PathBuf::from(env!("WASMEDGE_DIR"))
-            .join("bindings/rust/wasmedge-sys/tests/data/import.wasm");
+            .join("bindings/rust/wasmedge-sys/examples/data/import.wat");
 
         let result = Config::create();
         assert!(result.is_ok());
@@ -900,7 +918,7 @@ mod tests {
         assert!(config.bulk_memory_operations_enabled());
 
         // load module from file
-        let result = Loader::create(Some(config));
+        let result = Loader::create(Some(&config));
         assert!(result.is_ok());
         let loader = result.unwrap();
         let result = loader.from_file(path);
