@@ -25,6 +25,16 @@ namespace WASI {
 
 namespace {
 
+static __wasi_address_family_t *getAddressFamily(uint8_t *AddressBuf) {
+  return reinterpret_cast<__wasi_address_family_t *>(AddressBuf);
+}
+
+static uint8_t *getAddress(uint8_t *AddressBuf) {
+  // The first 2 bytes is for AddressFamily
+  const auto Diff = sizeof(uint16_t);
+  return &AddressBuf[Diff];
+}
+
 inline constexpr bool isSpecialFd(int Fd) noexcept {
   switch (Fd) {
   case STDIN_FILENO:
@@ -254,14 +264,16 @@ INode::fdFilestatGet(__wasi_filestat_t &Filestat) const noexcept {
     return WasiUnexpect(Res);
   }
 
-  Filestat.dev = Stat->st_dev;
-  Filestat.ino = Stat->st_ino;
+  // Zeroing out these values to prevent leaking information about the host
+  // environment from special fd such as stdin, stdout and stderr.
+  Filestat.dev = isSpecialFd(Fd) ? 0 : Stat->st_dev;
+  Filestat.ino = isSpecialFd(Fd) ? 0 : Stat->st_ino;
   Filestat.filetype = unsafeFiletype();
-  Filestat.nlink = Stat->st_nlink;
-  Filestat.size = Stat->st_size;
-  Filestat.atim = fromTimespec(Stat->st_atimespec);
-  Filestat.mtim = fromTimespec(Stat->st_mtimespec);
-  Filestat.ctim = fromTimespec(Stat->st_ctimespec);
+  Filestat.nlink = isSpecialFd(Fd) ? 0 : Stat->st_nlink;
+  Filestat.size = isSpecialFd(Fd) ? 0 : Stat->st_size;
+  Filestat.atim = isSpecialFd(Fd) ? 0 : fromTimespec(Stat->st_atimespec);
+  Filestat.mtim = isSpecialFd(Fd) ? 0 : fromTimespec(Stat->st_mtimespec);
+  Filestat.ctim = isSpecialFd(Fd) ? 0 : fromTimespec(Stat->st_ctimespec);
 
   return {};
 }
@@ -436,6 +448,11 @@ WasiExpect<void> INode::fdRead(Span<Span<uint8_t>> IOVs,
   return {};
 }
 
+// Due to the unfortunate design of wasi::fd_readdir, It's nearly impossible to
+// provide a correct implementation. The below implementation is just a
+// workaround for most usages and may not be correct in some edge cases. The
+// readdir entry API is going to be updated to use a stream type, so we don't
+// have to deal with it right now.
 WasiExpect<void> INode::fdReaddir(Span<uint8_t> Buffer,
                                   __wasi_dircookie_t Cookie,
                                   __wasi_size_t &Size) noexcept {
@@ -450,9 +467,10 @@ WasiExpect<void> INode::fdReaddir(Span<uint8_t> Buffer,
     }
   }
 
-  if (unlikely(Cookie != Dir.Cookie)) {
-    Dir.Buffer.clear();
-    seekdir(Dir.Dir, Cookie);
+  if (Cookie == 0) {
+    ::rewinddir(Dir.Dir);
+  } else if (unlikely(Cookie != Dir.Cookie)) {
+    ::seekdir(Dir.Dir, Cookie);
   }
 
   Size = 0;
@@ -464,7 +482,7 @@ WasiExpect<void> INode::fdReaddir(Span<uint8_t> Buffer,
                 Buffer.begin());
       Buffer = Buffer.subspan(NewDataSize);
       Size += NewDataSize;
-      Dir.Buffer.erase(Dir.Buffer.begin(), Dir.Buffer.begin() + NewDataSize);
+      Dir.Buffer.clear();
       if (unlikely(Buffer.empty())) {
         break;
       }
@@ -478,7 +496,7 @@ WasiExpect<void> INode::fdReaddir(Span<uint8_t> Buffer,
       // End of entries
       break;
     }
-    Dir.Cookie = SysDirent->d_seekoff;
+    Dir.Cookie = ::telldir(Dir.Dir);
     std::string_view Name = SysDirent->d_name;
 
     Dir.Buffer.resize(sizeof(__wasi_dirent_t) + Name.size());
@@ -599,7 +617,8 @@ INode::pathFilestatSetTimes(std::string Path, __wasi_timestamp_t ATim,
       SysTimespec[1].tv_nsec = UTIME_OMIT;
     }
 
-    if (auto Res = ::utimensat(Fd, Path.c_str(), SysTimespec, 0);
+    if (auto Res =
+            ::utimensat(Fd, Path.c_str(), SysTimespec, AT_SYMLINK_NOFOLLOW);
         unlikely(Res != 0)) {
       return WasiUnexpect(fromErrNo(errno));
     }
@@ -624,7 +643,7 @@ INode::pathFilestatSetTimes(std::string Path, __wasi_timestamp_t ATim,
     NeedFile = true;
   }
 
-  FdHolder Target(::openat(Fd, Path.c_str(), O_RDONLY));
+  FdHolder Target(::openat(Fd, Path.c_str(), O_RDONLY | O_SYMLINK));
   if (unlikely(!Target.ok())) {
     return WasiUnexpect(fromErrNo(errno));
   }
@@ -804,9 +823,8 @@ WasiExpect<INode> INode::sockOpen(__wasi_address_family_t AddressFamily,
   }
 }
 
-WasiExpect<void> INode::sockBind(uint8_t *Address, uint8_t AddressLength,
-                                 uint16_t Port) noexcept {
-
+WasiExpect<void> INode::sockBindV1(uint8_t *Address, uint8_t AddressLength,
+                                   uint16_t Port) noexcept {
   if (AddressLength == 4) {
     struct sockaddr_in ServerAddr;
     ServerAddr.sin_family = AF_INET;
@@ -833,6 +851,58 @@ WasiExpect<void> INode::sockBind(uint8_t *Address, uint8_t AddressLength,
   return {};
 }
 
+WasiExpect<void> INode::sockBindV2(uint8_t *AddressBuf,
+                                   [[maybe_unused]] uint8_t AddressLength,
+                                   uint16_t Port) noexcept {
+  int AddrFamily;
+  uint8_t *Address;
+
+  if (AddressLength != 128) {
+    // Fallback
+    switch (AddressLength) {
+    case 4:
+      AddrFamily = AF_INET;
+      break;
+    case 16:
+      AddrFamily = AF_INET6;
+      break;
+    default:
+      return WasiUnexpect(__WASI_ERRNO_INVAL);
+    }
+    Address = AddressBuf;
+  } else {
+    AddrFamily = toAddressFamily(*getAddressFamily(AddressBuf));
+    Address = getAddress(AddressBuf);
+  }
+
+  struct sockaddr_in ServerAddr4 = {};
+  struct sockaddr_in6 ServerAddr6 = {};
+  struct sockaddr *ServerAddr = nullptr;
+  int RealSize = 0;
+
+  if (AddrFamily == AF_INET) {
+    ServerAddr = reinterpret_cast<struct sockaddr *>(&ServerAddr4);
+    RealSize = sizeof(ServerAddr4);
+
+    ServerAddr4.sin_family = AF_INET;
+    ServerAddr4.sin_port = htons(Port);
+    std::memcpy(&ServerAddr4.sin_addr, Address, sizeof(in_addr));
+  } else if (AddrFamily == AF_INET6) {
+    ServerAddr = reinterpret_cast<struct sockaddr *>(&ServerAddr6);
+    RealSize = sizeof(ServerAddr6);
+
+    ServerAddr6.sin6_family = AF_INET6;
+    ServerAddr6.sin6_port = htons(Port);
+    ServerAddr6.sin6_flowinfo = 0;
+    std::memcpy(&ServerAddr6.sin6_addr, Address, sizeof(in6_addr));
+  }
+
+  if (auto Res = ::bind(Fd, ServerAddr, RealSize); unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+  return {};
+}
+
 WasiExpect<void> INode::sockListen(int32_t Backlog) noexcept {
   if (auto Res = ::listen(Fd, Backlog); unlikely(Res < 0)) {
     return WasiUnexpect(fromErrNo(errno));
@@ -840,7 +910,7 @@ WasiExpect<void> INode::sockListen(int32_t Backlog) noexcept {
   return {};
 }
 
-WasiExpect<INode> INode::sockAccept() noexcept {
+WasiExpect<INode> INode::sockAcceptV1() noexcept {
   struct sockaddr_in ServerSocketAddr;
   ServerSocketAddr.sin_family = AF_INET;
   ServerSocketAddr.sin_addr.s_addr = INADDR_ANY;
@@ -857,8 +927,29 @@ WasiExpect<INode> INode::sockAccept() noexcept {
   }
 }
 
-WasiExpect<void> INode::sockConnect(uint8_t *Address, uint8_t AddressLength,
-                                    uint16_t Port) noexcept {
+WasiExpect<INode> INode::sockAcceptV2(__wasi_fdflags_t FdFlags) noexcept {
+  int NewFd;
+  if (NewFd = ::accept(Fd, nullptr, nullptr); unlikely(NewFd < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+
+  INode New(NewFd);
+  int SysFlag = fcntl(NewFd, F_GETFL, 0);
+  if (FdFlags) {
+    if (FdFlags & __WASI_FDFLAGS_NONBLOCK) {
+      SysFlag |= O_NONBLOCK;
+    }
+  }
+
+  if (auto Res = ::fcntl(Fd, F_SETFL, SysFlag); unlikely(Res != 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  } else {
+    return New;
+  }
+}
+
+WasiExpect<void> INode::sockConnectV1(uint8_t *Address, uint8_t AddressLength,
+                                      uint16_t Port) noexcept {
   if (AddressLength == 4) {
     struct sockaddr_in ClientSocketAddr;
     ClientSocketAddr.sin_family = AF_INET;
@@ -887,17 +978,69 @@ WasiExpect<void> INode::sockConnect(uint8_t *Address, uint8_t AddressLength,
   return {};
 }
 
+WasiExpect<void> INode::sockConnectV2(uint8_t *AddressBuf,
+                                      [[maybe_unused]] uint8_t AddressLength,
+                                      uint16_t Port) noexcept {
+  int AddrFamily;
+  uint8_t *Address;
+
+  if (AddressLength != 128) {
+    // Fallback
+    switch (AddressLength) {
+    case 4:
+      AddrFamily = AF_INET;
+      break;
+    case 16:
+      AddrFamily = AF_INET6;
+      break;
+    default:
+      return WasiUnexpect(__WASI_ERRNO_INVAL);
+    }
+    Address = AddressBuf;
+  } else {
+    AddrFamily = toAddressFamily(*getAddressFamily(AddressBuf));
+    Address = getAddress(AddressBuf);
+  }
+
+  struct sockaddr_in ClientAddr4 {};
+  struct sockaddr_in6 ClientAddr6 {};
+  struct sockaddr *ClientAddr = nullptr;
+  int RealSize = 0;
+
+  if (AddrFamily == AF_INET) {
+    ClientAddr = reinterpret_cast<struct sockaddr *>(&ClientAddr4);
+    RealSize = sizeof(ClientAddr4);
+
+    ClientAddr4.sin_family = AF_INET;
+    ClientAddr4.sin_port = htons(Port);
+    std::memcpy(&ClientAddr4.sin_addr, Address, sizeof(in_addr));
+  } else if (AddrFamily == AF_INET6) {
+    ClientAddr = reinterpret_cast<struct sockaddr *>(&ClientAddr6);
+    RealSize = sizeof(ClientAddr6);
+
+    ClientAddr6.sin6_family = AF_INET6;
+    ClientAddr6.sin6_flowinfo = 0;
+    ClientAddr6.sin6_port = htons(Port);
+    std::memcpy(&ClientAddr6.sin6_addr, Address, sizeof(in6_addr));
+  }
+
+  if (auto Res = ::connect(Fd, ClientAddr, RealSize); unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+  return {};
+}
+
 WasiExpect<void> INode::sockRecv(Span<Span<uint8_t>> RiData,
                                  __wasi_riflags_t RiFlags, __wasi_size_t &NRead,
                                  __wasi_roflags_t &RoFlags) const noexcept {
-  return sockRecvFrom(RiData, RiFlags, nullptr, 0, NRead, RoFlags);
+  return sockRecvFromV1(RiData, RiFlags, nullptr, 0, NRead, RoFlags);
 }
 
-WasiExpect<void> INode::sockRecvFrom(Span<Span<uint8_t>> RiData,
-                                     __wasi_riflags_t RiFlags, uint8_t *Address,
-                                     uint8_t AddressLength,
-                                     __wasi_size_t &NRead,
-                                     __wasi_roflags_t &RoFlags) const noexcept {
+WasiExpect<void>
+INode::sockRecvFromV1(Span<Span<uint8_t>> RiData, __wasi_riflags_t RiFlags,
+                      uint8_t *Address, uint8_t AddressLength,
+                      __wasi_size_t &NRead,
+                      __wasi_roflags_t &RoFlags) const noexcept {
   int SysRiFlags = 0;
   if (RiFlags & __WASI_RIFLAGS_RECV_PEEK) {
     SysRiFlags |= MSG_PEEK;
@@ -956,6 +1099,91 @@ WasiExpect<void> INode::sockRecvFrom(Span<Span<uint8_t>> RiData,
   return {};
 }
 
+WasiExpect<void> INode::sockRecvFromV2(
+    Span<Span<uint8_t>> RiData, __wasi_riflags_t RiFlags, uint8_t *AddressBuf,
+    [[maybe_unused]] uint8_t AddressLength, uint32_t *PortPtr,
+    __wasi_size_t &NRead, __wasi_roflags_t &RoFlags) const noexcept {
+  uint8_t *Address = nullptr;
+  __wasi_address_family_t *AddrFamily = nullptr;
+  __wasi_address_family_t Dummy; // Write garbage on fallback mode.
+
+  if (AddressBuf) {
+    if (AddressLength != 128) {
+      // Fallback
+      AddrFamily = &Dummy;
+      Address = AddressBuf;
+    } else {
+      AddrFamily = getAddressFamily(AddressBuf);
+      Address = getAddress(AddressBuf);
+    }
+  }
+
+  int SysRiFlags = 0;
+  if (RiFlags & __WASI_RIFLAGS_RECV_PEEK) {
+    SysRiFlags |= MSG_PEEK;
+  }
+  if (RiFlags & __WASI_RIFLAGS_RECV_WAITALL) {
+    SysRiFlags |= MSG_WAITALL;
+  }
+
+  iovec SysIOVs[kIOVMax];
+  size_t SysIOVsSize = 0;
+  for (auto &IOV : RiData) {
+    SysIOVs[SysIOVsSize].iov_base = IOV.data();
+    SysIOVs[SysIOVsSize].iov_len = IOV.size();
+    ++SysIOVsSize;
+  }
+
+  sockaddr_storage SockAddrStorage;
+  int MaxAllowLength = sizeof(SockAddrStorage);
+
+  msghdr SysMsgHdr;
+  SysMsgHdr.msg_name = &SockAddrStorage;
+  SysMsgHdr.msg_namelen = MaxAllowLength;
+  SysMsgHdr.msg_iov = SysIOVs;
+  SysMsgHdr.msg_iovlen = SysIOVsSize;
+  SysMsgHdr.msg_control = nullptr;
+  SysMsgHdr.msg_controllen = 0;
+  SysMsgHdr.msg_flags = 0;
+
+  // Store recv bytes length and flags.
+  if (auto Res = ::recvmsg(Fd, &SysMsgHdr, SysRiFlags); unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  } else {
+    NRead = Res;
+  }
+
+  if (AddressBuf) {
+    *AddrFamily = fromAddressFamily(SockAddrStorage.ss_family);
+    if (SockAddrStorage.ss_family == AF_INET) {
+      std::memcpy(Address,
+                  &reinterpret_cast<sockaddr_in *>(&SockAddrStorage)->sin_addr,
+                  sizeof(in_addr));
+    } else if (SockAddrStorage.ss_family == AF_INET6) {
+      std::memcpy(
+          Address,
+          &reinterpret_cast<sockaddr_in6 *>(&SockAddrStorage)->sin6_addr,
+          sizeof(in6_addr));
+    }
+  }
+
+  if (PortPtr) {
+    *AddrFamily = fromAddressFamily(SockAddrStorage.ss_family);
+    if (SockAddrStorage.ss_family == AF_INET) {
+      *PortPtr = reinterpret_cast<sockaddr_in *>(&SockAddrStorage)->sin_port;
+    } else if (SockAddrStorage.ss_family == AF_INET6) {
+      *PortPtr = reinterpret_cast<sockaddr_in6 *>(&SockAddrStorage)->sin6_port;
+    }
+  }
+
+  RoFlags = static_cast<__wasi_roflags_t>(0);
+  if (SysMsgHdr.msg_flags & MSG_TRUNC) {
+    RoFlags |= __WASI_ROFLAGS_RECV_DATA_TRUNCATED;
+  }
+
+  return {};
+}
+
 WasiExpect<void> INode::sockSend(Span<Span<const uint8_t>> SiData,
                                  __wasi_siflags_t SiFlags,
                                  __wasi_size_t &NWritten) const noexcept {
@@ -963,34 +1191,58 @@ WasiExpect<void> INode::sockSend(Span<Span<const uint8_t>> SiData,
 }
 
 WasiExpect<void> INode::sockSendTo(Span<Span<const uint8_t>> SiData,
-                                   __wasi_siflags_t, uint8_t *Address,
-                                   uint8_t AddressLength, int32_t Port,
+                                   __wasi_siflags_t, uint8_t *AddressBuf,
+                                   [[maybe_unused]] uint8_t AddressLength,
+                                   int32_t Port,
                                    __wasi_size_t &NWritten) const noexcept {
 #ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
 #endif
   int SysSiFlags = MSG_NOSIGNAL;
+  uint8_t *Address = nullptr;
+  int AddrFamily = 0;
+
+  if (AddressBuf) {
+    if (AddressLength != 128) {
+      // Fallback
+      switch (AddressLength) {
+      case 4:
+        AddrFamily = AF_INET;
+        break;
+      case 16:
+        AddrFamily = AF_INET6;
+        break;
+      default:
+        return WasiUnexpect(__WASI_ERRNO_INVAL);
+      }
+      Address = AddressBuf;
+    } else {
+      AddrFamily = toAddressFamily(*getAddressFamily(AddressBuf));
+      Address = getAddress(AddressBuf);
+    }
+  }
 
   void *MsgName = nullptr;
   socklen_t MsgNameLen = 0;
-  struct sockaddr_in ClientSocketAddr;
-  struct sockaddr_in6 ClientSocketAddr6;
+  struct sockaddr_in ClientAddr4 = {};
+  struct sockaddr_in6 ClientAddr6 = {};
 
   if (Address) {
-    if (AddressLength == 4) {
-      ClientSocketAddr.sin_family = AF_INET;
-      ClientSocketAddr.sin_port = htons(Port);
-      std::memcpy(&ClientSocketAddr.sin_addr.s_addr, Address, AddressLength);
+    if (AddrFamily == AF_INET) {
+      MsgName = &ClientAddr4;
+      MsgNameLen = sizeof(ClientAddr4);
 
-      MsgName = &ClientSocketAddr;
-      MsgNameLen = sizeof(ClientSocketAddr);
-    } else if (AddressLength == 16) {
-      ClientSocketAddr6.sin6_family = AF_INET6;
-      ClientSocketAddr6.sin6_port = htons(Port);
-      std::memcpy(&ClientSocketAddr6.sin6_addr.s6_addr, Address, AddressLength);
+      ClientAddr4.sin_family = AF_INET;
+      ClientAddr4.sin_port = htons(Port);
+      std::memcpy(&ClientAddr4.sin_addr, Address, sizeof(in_addr));
+    } else if (AddrFamily == AF_INET6) {
+      MsgName = &ClientAddr6;
+      MsgNameLen = sizeof(ClientAddr6);
 
-      MsgName = &ClientSocketAddr6;
-      MsgNameLen = sizeof(ClientSocketAddr6);
+      ClientAddr6.sin6_family = AF_INET6;
+      ClientAddr6.sin6_flowinfo = 0;
+      ClientAddr6.sin6_port = htons(Port);
+      std::memcpy(&ClientAddr6.sin6_addr, Address, sizeof(in6_addr));
     }
   }
 
@@ -1079,14 +1331,145 @@ WasiExpect<void> INode::sockSetOpt(__wasi_sock_opt_level_t SockOptLevel,
   return {};
 }
 
-WasiExpect<void> INode::sockGetLoaclAddr(uint8_t *, uint32_t *,
-                                         uint32_t *) const noexcept {
-  return WasiUnexpect(__WASI_ERRNO_NOSYS);
+WasiExpect<void> INode::sockGetLocalAddrV1(uint8_t *AddressPtr,
+                                           uint32_t *AddrTypePtr,
+                                           uint32_t *PortPtr) const noexcept {
+  struct sockaddr_storage SocketAddr;
+  socklen_t Slen = sizeof(SocketAddr);
+  std::memset(&SocketAddr, 0, sizeof(SocketAddr));
+
+  if (auto Res =
+          ::getsockname(Fd, reinterpret_cast<sockaddr *>(&SocketAddr), &Slen);
+      unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+
+  auto AddrLen = 4;
+  if (Slen != 16) {
+    AddrLen = 16;
+  }
+
+  if (SocketAddr.ss_family == AF_INET) {
+    *AddrTypePtr = 4;
+    auto SocketAddrv4 = reinterpret_cast<struct sockaddr_in *>(&SocketAddr);
+
+    *PortPtr = ntohs(SocketAddrv4->sin_port);
+    std::memcpy(AddressPtr, &(SocketAddrv4->sin_addr.s_addr), AddrLen);
+  } else if (SocketAddr.ss_family == AF_INET6) {
+    *AddrTypePtr = 6;
+    auto SocketAddrv6 = reinterpret_cast<struct sockaddr_in6 *>(&SocketAddr);
+
+    *PortPtr = ntohs(SocketAddrv6->sin6_port);
+    std::memcpy(AddressPtr, SocketAddrv6->sin6_addr.s6_addr, AddrLen);
+  } else {
+    return WasiUnexpect(__WASI_ERRNO_NOSYS);
+  }
+
+  return {};
 }
 
-WasiExpect<void> INode::sockGetPeerAddr(uint8_t *, uint32_t *,
-                                        uint32_t *) const noexcept {
-  return WasiUnexpect(__WASI_ERRNO_NOSYS);
+WasiExpect<void> INode::sockGetLocalAddrV2(uint8_t *AddressBufPtr,
+                                           uint32_t *PortPtr) const noexcept {
+  auto AddrFamilyPtr = getAddressFamily(AddressBufPtr);
+  auto AddressPtr = getAddress(AddressBufPtr);
+
+  struct sockaddr_storage SocketAddr;
+  socklen_t Slen = sizeof(SocketAddr);
+  std::memset(&SocketAddr, 0, sizeof(SocketAddr));
+
+  if (auto Res =
+          ::getsockname(Fd, reinterpret_cast<sockaddr *>(&SocketAddr), &Slen);
+      unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+
+  if (SocketAddr.ss_family == AF_INET) {
+    auto SocketAddrv4 = reinterpret_cast<struct sockaddr_in *>(&SocketAddr);
+
+    *AddrFamilyPtr = fromAddressFamily(AF_INET);
+    *PortPtr = ntohs(SocketAddrv4->sin_port);
+    std::memcpy(AddressPtr, &SocketAddrv4->sin_addr, sizeof(in_addr));
+  } else if (SocketAddr.ss_family == AF_INET6) {
+    auto SocketAddrv6 = reinterpret_cast<struct sockaddr_in6 *>(&SocketAddr);
+
+    *AddrFamilyPtr = fromAddressFamily(AF_INET6);
+    *PortPtr = ntohs(SocketAddrv6->sin6_port);
+    std::memcpy(AddressPtr, &SocketAddrv6->sin6_addr, sizeof(in6_addr));
+  } else {
+    return WasiUnexpect(__WASI_ERRNO_NOSYS);
+  }
+
+  return {};
+}
+
+WasiExpect<void> INode::sockGetPeerAddrV1(uint8_t *AddressPtr,
+                                          uint32_t *AddrTypePtr,
+                                          uint32_t *PortPtr) const noexcept {
+  struct sockaddr_storage SocketAddr;
+  socklen_t Slen = sizeof(SocketAddr);
+  std::memset(&SocketAddr, 0, sizeof(SocketAddr));
+
+  if (auto Res =
+          ::getpeername(Fd, reinterpret_cast<sockaddr *>(&SocketAddr), &Slen);
+      unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+
+  auto AddrLen = 4;
+  if (Slen != 16) {
+    AddrLen = 16;
+  }
+
+  if (SocketAddr.ss_family == AF_INET) {
+    *AddrTypePtr = 4;
+    auto SocketAddrv4 = reinterpret_cast<struct sockaddr_in *>(&SocketAddr);
+
+    *PortPtr = ntohs(SocketAddrv4->sin_port);
+    std::memcpy(AddressPtr, &(SocketAddrv4->sin_addr.s_addr), AddrLen);
+  } else if (SocketAddr.ss_family == AF_INET6) {
+    *AddrTypePtr = 6;
+    auto SocketAddrv6 = reinterpret_cast<struct sockaddr_in6 *>(&SocketAddr);
+    *PortPtr = ntohs(SocketAddrv6->sin6_port);
+    std::memcpy(AddressPtr, SocketAddrv6->sin6_addr.s6_addr, AddrLen);
+  } else {
+    return WasiUnexpect(__WASI_ERRNO_NOSYS);
+  }
+
+  return {};
+}
+
+WasiExpect<void> INode::sockGetPeerAddrV2(uint8_t *AddressBufPtr,
+                                          uint32_t *PortPtr) const noexcept {
+  auto AddrFamilyPtr = getAddressFamily(AddressBufPtr);
+  auto AddressPtr = getAddress(AddressBufPtr);
+
+  struct sockaddr_storage SocketAddr;
+  socklen_t Slen = sizeof(SocketAddr);
+  std::memset(&SocketAddr, 0, sizeof(SocketAddr));
+
+  if (auto Res =
+          ::getpeername(Fd, reinterpret_cast<sockaddr *>(&SocketAddr), &Slen);
+      unlikely(Res < 0)) {
+    return WasiUnexpect(fromErrNo(errno));
+  }
+
+  if (SocketAddr.ss_family == AF_INET) {
+    auto SocketAddrv4 = reinterpret_cast<struct sockaddr_in *>(&SocketAddr);
+
+    *AddrFamilyPtr = fromAddressFamily(AF_INET);
+    *PortPtr = ntohs(SocketAddrv4->sin_port);
+    std::memcpy(AddressPtr, &SocketAddrv4->sin_addr, sizeof(in_addr));
+  } else if (SocketAddr.ss_family == AF_INET6) {
+    auto SocketAddrv6 = reinterpret_cast<struct sockaddr_in6 *>(&SocketAddr);
+
+    *AddrFamilyPtr = fromAddressFamily(AF_INET6);
+    *PortPtr = ntohs(SocketAddrv6->sin6_port);
+    std::memcpy(AddressPtr, &SocketAddrv6->sin6_addr, sizeof(in6_addr));
+  } else {
+    return WasiUnexpect(__WASI_ERRNO_NOSYS);
+  }
+
+  return {};
 }
 
 __wasi_filetype_t INode::unsafeFiletype() const noexcept {
@@ -1324,7 +1707,7 @@ WasiExpect<void> INode::getAddrinfo(std::string_view Node,
   SysHint.ai_flags = toAIFlags(Hint.ai_flags);
   SysHint.ai_family = toAddressFamily(Hint.ai_family);
   SysHint.ai_socktype = toSockType(Hint.ai_socktype);
-  SysHint.ai_protocol = toProtocal(Hint.ai_protocol);
+  SysHint.ai_protocol = toProtocol(Hint.ai_protocol);
   SysHint.ai_addrlen = 0;
   SysHint.ai_addr = nullptr;
   SysHint.ai_canonname = nullptr;
@@ -1348,7 +1731,7 @@ WasiExpect<void> INode::getAddrinfo(std::string_view Node,
     auto &CurAddrinfo = WasiAddrinfoArray[Idx];
     CurAddrinfo->ai_flags = fromAIFlags(SysResItem->ai_flags);
     CurAddrinfo->ai_socktype = fromSockType(SysResItem->ai_socktype);
-    CurAddrinfo->ai_protocol = fromProtocal(SysResItem->ai_protocol);
+    CurAddrinfo->ai_protocol = fromProtocol(SysResItem->ai_protocol);
     CurAddrinfo->ai_family = fromAddressFamily(SysResItem->ai_family);
     CurAddrinfo->ai_addrlen = SysResItem->ai_addrlen;
 
